@@ -4,15 +4,15 @@ import six
 import time
 import re
 
-from . import pyrebase
+from . import logs
 from .auth import get_auth
-from .http_artifact_store import HTTPArtifactStore
+from .credentials import Credentials
+from .model_setup import get_model_verbose_level
+from .http_storage_handler import HTTPStorageHandler
 from .experiment import experiment_from_dict
+from .tartifact_store import TartifactStore
 from .util import retry
 
-import logging
-logging.basicConfig()
-
 
 class HTTPProvider(object):
     """Data provider communicating with API server."""
@@ -25,19 +25,26 @@ class HTTPProvider(object):
             compression=None):
         # TODO: implement connection
         self.url = config.get('serverUrl')
-        self.verbose = verbose
-        self.logger = logging.getLogger('HTTPProvider')
+        self.verbose = get_model_verbose_level()
+        self.logger = logs.getLogger('HTTPProvider')
         self.logger.setLevel(self.verbose)
 
+        self.credentials: Credentials = \
+            Credentials.getCredentials(config)
+
+        self.storage_handler = HTTPStorageHandler(
+            self.url,
+            self.credentials.to_dict() if self.credentials else None,
+            compression=compression)
+        self.store = TartifactStore(self.storage_handler, self.logger)
+
         self.auth = None
-        self.app = pyrebase.initialize_app(config)
         guest = config.get('guest')
         if not guest and 'serviceAccount' not in config.keys():
-            self.auth = get_auth(self.app,
-                                 config.get("use_email_auth"),
-                                 config.get("email"),
-                                 config.get("password"),
-                                 blocking_auth)
+            self.auth = get_auth(
+                config.get('authentication'),
+                blocking_auth
+            )
 
         self.compression = compression
         if self.compression is None:
@@ -50,11 +57,8 @@ class HTTPProvider(object):
         compression = compression if compression else self.compression
 
         for tag, art in six.iteritems(experiment.artifacts):
-            if not art['mutable'] and art.get('local') is not None:
-                art['hash'] = HTTPArtifactStore(None, None,
-                                                compression,
-                                                self.verbose) \
-                    .get_artifact_hash(art)
+            if not art.is_mutable and art.local_path is not None:
+                art['hash'] = self.store.get_artifact_hash(art)
 
         data = {}
         data['experiment'] = experiment.__dict__
@@ -69,7 +73,12 @@ class HTTPProvider(object):
             self._raise_detailed_error(request)
             return request
 
-        request = retry(post_request, sleep_time=10, logger=self.logger)
+        request = retry(
+            post_request,
+            no_retries=10,
+            sleep_time=10,
+            logger=self.logger)
+
         artifacts = request.json()['artifacts']
 
         self._update_artifacts(experiment, artifacts)
@@ -81,16 +90,11 @@ class HTTPProvider(object):
         for tag, art in six.iteritems(experiment.artifacts):
             target_art = artifacts.get(tag)
             if target_art is not None:
-                art['key'] = target_art['key']
-                art['qualified'] = target_art['qualified']
-                art['bucket'] = target_art['bucket']
+                art.key = target_art.key
+                art.remote_path = target_art.remote_path
 
-                if art.get('local'):
-                    HTTPArtifactStore(target_art['url'],
-                                      target_art['timestamp'],
-                                      compression=self.compression,
-                                      verbose=self.verbose) \
-                        .put_artifact(art)
+                if art.local_path is not None:
+                    self.store.put_artifact(art)
 
     def delete_experiment(self, experiment):
         if isinstance(experiment, six.string_types):
@@ -124,8 +128,10 @@ class HTTPProvider(object):
                                     )
 
             self._raise_detailed_error(request)
-            return experiment_from_dict(request.json()['experiment'])
+            data = request.json()['experiment']
+            return experiment_from_dict(data)
         except BaseException as e:
+            self.logger.info('error getting experiment {}'.format(key))
             self.logger.info(e)
             return None
 
@@ -238,11 +244,7 @@ class HTTPProvider(object):
             experiment = self.get_experiment(experiment_key)
             artifact = experiment.artifacts[artifact_tag]
 
-        return HTTPArtifactStore(
-            artifact.get('url'),
-            timestamp=time.time(),
-            verbose=self.verbose
-        ).get_artifact(artifact, local_path=local_path)
+        return self.store.get_artifact(artifact, local_path=local_path)
 
     def get_users(self):
         headers = self._get_headers()
