@@ -1,36 +1,24 @@
 import sys
 import argparse
-import json
 import re
 import os
 import uuid
 import importlib
 import time
 import six
-import traceback
 import numpy as np
 
 from datetime import timedelta
 
-from .local_queue import LocalQueue
-from .pubsub_queue import PubsubQueue
-from .sqs_queue import SQSQueue
-from .qclient_cache import get_cached_queue, shutdown_cached_queue
-from .gcloud_worker import GCloudWorkerManager
-from .ec2cloud_worker import EC2WorkerManager
 from .hyperparameter import HyperparameterParser
 from .util import rand_string, Progbar, rsync_cp
 from .experiment import create_experiment
-from .unencrypted_payload_builder import UnencryptedPayloadBuilder
-from .encrypted_payload_builder import EncryptedPayloadBuilder
-
+from .experiment_submitter import submit_experiments
 from . import model
 from . import git_util
 from . import local_worker
 from . import fs_tracker
 from . import logs
-from .auth import get_auth
-
 
 def main(args=sys.argv[1:]):
     logger = logs.getLogger('studio-runner')
@@ -276,15 +264,15 @@ def main(args=sys.argv[1:]):
                 'Specify --force-git to run experiment from dirty git repo')
             sys.exit(1)
 
-    resources_needed = parse_hardware(runner_args, config['resources_needed'])
+    resources_needed = _parse_hardware(runner_args, config['resources_needed'])
     logger.debug('resources requested: ')
     logger.debug(str(resources_needed))
 
     artifacts = {}
-    artifacts.update(parse_artifacts(runner_args.capture, mutable=True))
-    artifacts.update(parse_artifacts(runner_args.capture_once, mutable=False))
+    artifacts.update(_parse_artifacts(runner_args.capture, mutable=True))
+    artifacts.update(_parse_artifacts(runner_args.capture_once, mutable=False))
     with model.get_db_provider(config) as db:
-        artifacts.update(parse_external_artifacts(runner_args.reuse, db))
+        artifacts.update(_parse_external_artifacts(runner_args.reuse, db))
 
     if runner_args.branch:
         config['cloud']['branch'] = runner_args.branch
@@ -296,9 +284,11 @@ def main(args=sys.argv[1:]):
     if runner_args.lifetime:
         config['experimentLifetime'] = runner_args.lifetime
 
+    queueLifetime = None
+
     if any(runner_args.hyperparam):
-        if runner_args.optimizer is "grid":
-            experiments = add_hyperparam_experiments(
+        if runner_args.optimizer == "grid":
+            experiments = _add_hyperparam_experiments(
                 exec_filename,
                 other_args,
                 runner_args,
@@ -306,12 +296,19 @@ def main(args=sys.argv[1:]):
                 resources_needed,
                 logger)
 
+            queue = model.get_queue(
+                queue_name=runner_args.queue,
+                cloud=runner_args.cloud,
+                config=config,
+                close_after=queueLifetime,
+                logger=logger,
+                verbose=verbose)
+
             queue_name = submit_experiments(
                 experiments,
                 config=config,
                 logger=logger,
-                queue_name=runner_args.queue,
-                cloud=runner_args.cloud)
+                queue=queue)
 
             spin_up_workers(
                 runner_args,
@@ -349,7 +346,7 @@ def main(args=sys.argv[1:]):
                 hyperparam_pop = optimizer.ask()
                 hyperparam_tuples = h.convert_to_tuples(hyperparam_pop)
 
-                experiments = add_hyperparam_experiments(
+                experiments = _add_hyperparam_experiments(
                     exec_filename,
                     other_args,
                     runner_args,
@@ -359,12 +356,19 @@ def main(args=sys.argv[1:]):
                     optimizer=optimizer,
                     hyperparam_tuples=hyperparam_tuples)
 
+                queue = model.get_queue(
+                    queue_name=queue_name,
+                    cloud=runner_args.cloud,
+                    config=config,
+                    close_after=queueLifetime,
+                    logger=logger,
+                    verbose=verbose)
+
                 queue_name = submit_experiments(
                     experiments,
                     config=config,
                     logger=logger,
-                    cloud=runner_args.cloud,
-                    queue_name=queue_name)
+                    queue=queue)
 
                 if not workers_started:
                     spin_up_workers(
@@ -378,10 +382,6 @@ def main(args=sys.argv[1:]):
                 fitnesses, behaviors = get_experiment_fitnesses(
                     experiments, optimizer, config, logger)
 
-                # for i, hh in enumerate(hyperparam_pop):
-                #     print fitnesses[i]
-                #     for hhh in hh:
-                #         print hhh
                 try:
                     optimizer.tell(hyperparam_pop, fitnesses, behaviors)
                 except BaseException:
@@ -415,12 +415,19 @@ def main(args=sys.argv[1:]):
                 max_duration=runner_args.max_duration,
             )]
 
+        queue = model.get_queue(
+            queue_name=runner_args.queue,
+            cloud=runner_args.cloud,
+            config=config,
+            close_after=queueLifetime,
+            logger=logger,
+            verbose=verbose)
+
         queue_name = submit_experiments(
             experiments,
             config=config,
             logger=logger,
-            cloud=runner_args.cloud,
-            queue_name=runner_args.queue)
+            queue=queue)
 
         spin_up_workers(
             runner_args,
@@ -431,95 +438,6 @@ def main(args=sys.argv[1:]):
 
     return
 
-
-def add_experiment(args):
-    try:
-        config, python_pkg, e = args
-
-        e.pythonenv = add_packages(e.pythonenv, python_pkg)
-
-        with model.get_db_provider(config) as db:
-            db.add_experiment(e)
-    except BaseException:
-        traceback.print_exc()
-        raise
-    return e
-
-
-def get_worker_manager(config, cloud=None, verbose=10):
-    if cloud is None:
-        return None
-
-    assert cloud in ['gcloud', 'gcspot', 'ec2', 'ec2spot']
-    logger = logs.getLogger('runner.get_worker_manager')
-    logger.setLevel(verbose)
-
-    auth = get_auth(config['database']['authentication'])
-    auth_cookie = auth.get_token_file() if auth else None
-
-    branch = config['cloud'].get('branch')
-
-    logger.info('using branch {}'.format(branch))
-
-    if cloud in ['gcloud', 'gcspot']:
-
-        cloudconfig = config['cloud']['gcloud']
-        worker_manager = GCloudWorkerManager(
-            auth_cookie=auth_cookie,
-            zone=cloudconfig['zone'],
-            branch=branch,
-            user_startup_script=config['cloud'].get('user_startup_script')
-        )
-
-    if cloud in ['ec2', 'ec2spot']:
-        worker_manager = EC2WorkerManager(
-            auth_cookie=auth_cookie,
-            branch=branch,
-            user_startup_script=config['cloud'].get('user_startup_script')
-        )
-    return worker_manager
-
-
-def get_queue(
-        queue_name=None,
-        cloud=None,
-        config=None,
-        logger=None,
-        close_after=None,
-        verbose=10):
-    if queue_name is None:
-        if cloud in ['gcloud', 'gcspot']:
-            queue_name = 'pubsub_' + str(uuid.uuid4())
-        elif cloud in ['ec2', 'ec2spot']:
-            queue_name = 'sqs_' + str(uuid.uuid4())
-        else:
-            queue_name = 'local'
-
-    if queue_name.startswith('ec2') or \
-       queue_name.startswith('sqs'):
-        return SQSQueue(queue_name, verbose=verbose)
-    elif queue_name.startswith('rmq_'):
-        return get_cached_queue(
-            name=queue_name,
-            route='StudioML.' + queue_name,
-            config=config,
-            close_after=close_after,
-            logger=logger,
-            verbose=verbose)
-    elif queue_name == 'local':
-        return LocalQueue(verbose=verbose)
-    else:
-        return PubsubQueue(queue_name, verbose=verbose)
-
-def shutdown_queue(queue, logger=None, delete_queue=True):
-    if queue is None:
-        return
-    queue_name = queue.get_name()
-    if queue_name.startswith("rmq_"):
-        shutdown_cached_queue(queue, logger, delete_queue)
-    else:
-        queue.shutdown(delete_queue)
-
 def spin_up_workers(
         runner_args,
         config,
@@ -530,7 +448,7 @@ def spin_up_workers(
     if cloud:
 
         assert cloud in ['gcloud', 'gcspot', 'ec2', 'ec2spot']
-        worker_manager = get_worker_manager(config, cloud, verbose=verbose)
+        worker_manager = model.get_worker_manager(config, cloud, verbose=verbose)
 
         if cloud == 'gcloud' or \
            cloud == 'ec2':
@@ -584,67 +502,6 @@ def spin_up_workers(
                                       "implemented yet")
 
 
-def submit_experiments(
-        experiments,
-        config,
-        logger,
-        cloud=None,
-        queue_name=None,
-        python_pkg=[]):
-
-    num_experiments = len(experiments)
-    verbose = model.parse_verbosity(config['verbose'])
-
-    payload_builder = UnencryptedPayloadBuilder("simple-payload")
-    # Are we using experiment payload encryption?
-    key_path = config.get('public_key_path')
-    if key_path is not None:
-        logger.info("Using RSA public key path: {0}".format(key_path))
-        payload_builder = \
-            EncryptedPayloadBuilder(
-                "cs-rsa-encryptor [{0}]".format(key_path), key_path)
-
-    start_time = time.time()
-
-    # Update Python environment info for our experiments:
-    for experiment in experiments:
-        experiment.pythonenv = add_packages(experiment.pythonenv, python_pkg)
-
-    # Now add them to experiments database:
-    try:
-        with model.get_db_provider(config) as db:
-            for experiment in experiments:
-                db.add_experiment(experiment)
-    except BaseException:
-        traceback.print_exc()
-        raise
-
-    experiments = [add_experiment(e) for e in
-                   zip([config] * num_experiments,
-                       [python_pkg] *
-                       num_experiments,
-                       experiments)]
-
-
-    queue = get_queue(
-        queue_name=queue_name,
-        cloud=cloud,
-        config=config,
-        close_after=timedelta(
-            minutes=2),
-        logger=logger,
-        verbose=verbose)
-
-    for experiment in experiments:
-        payload = payload_builder.construct(experiment, config, python_pkg)
-        queue.enqueue(json.dumps(payload))
-        logger.info("studio run: submitted experiment " + experiment.key)
-
-    logger.info("Added {0} experiment(s) in {1} seconds to queue {2}"
-                .format(num_experiments, int(time.time() - start_time), queue_name))
-    return queue.get_name()
-
-
 def get_experiment_fitnesses(experiments, optimizer, config, logger):
     with model.get_db_provider() as db:
         progbar = Progbar(len(experiments), interval=0.0)
@@ -745,11 +602,10 @@ def get_experiment_fitnesses(experiments, optimizer, config, logger):
                             break
 
             time.sleep(config['sleep_time'])
-        print
         return fitnesses, behaviors
 
 
-def parse_artifacts(art_list, mutable):
+def _parse_artifacts(art_list, mutable):
     retval = {}
     url_schema = re.compile('^https{0,1}://')
     s3_schema = re.compile('s3://')
@@ -792,7 +648,7 @@ def parse_artifacts(art_list, mutable):
     return retval
 
 
-def parse_external_artifacts(art_list, db):
+def _parse_external_artifacts(art_list, db):
     retval = {}
     for entry in art_list:
         external = re.sub(':.*', '', entry)
@@ -803,13 +659,13 @@ def parse_external_artifacts(art_list, db):
         experiment = db.get_experiment(experiment_key, getinfo=False)
 
         retval[tag] = {
-            'key': experiment.artifacts[external_tag]['key'],
+            'key': experiment.artifacts[external_tag].key,
             'mutable': False
         }
     return retval
 
 
-def parse_hardware(runner_args, config={}):
+def _parse_hardware(runner_args, config={}):
     resources_needed = {}
     parse_list = ['gpus', 'cpus', 'ram', 'hdd', 'gpuMem']
     for key in parse_list:
@@ -823,7 +679,7 @@ def parse_hardware(runner_args, config={}):
     return resources_needed
 
 
-def add_hyperparam_experiments(
+def _add_hyperparam_experiments(
         exec_filename,
         other_args,
         runner_args,
@@ -915,20 +771,5 @@ def add_hyperparam_experiments(
 
     return experiments
 
-
-def add_packages(list1, list2):
-    # This function dedups the package names which I think could be
-    # functionally not desirable however rather than changing the behavior
-    # instead we will do the dedup in a stable manner that prevents
-    # package re-ordering
-    pkgs = {re.sub('==.+', '', pkg): pkg for pkg in list1 + list2}
-    merged = []
-    for k in list1 + list2:
-        v = pkgs.pop(re.sub('==.+', '', k), None)
-        if v is not None:
-            merged.append(v)
-    return merged
-
-
 if __name__ == "__main__":
     main()
